
/**
 * Zone Manager
 *
 * @package zonemanager
 * @version 2.0
 * @author Bonus Themes
 * @copyright Bonus Themes 2008
 * @license GNU/GPL
 * @link http://www.bonusthemes.com/goto/zonemanager
 */

function Zone(idx, target)
{
	this.idx = idx;
	this.wdg = document.createElement('div');
	this.wdg.className = 'zone';

	if (target == null) {
		var zones_host = document.getElementById('zones_host');
		zones_host.appendChild(this.wdg);
		zones_host.appendChild(mkgap());
	}
	else {
		target.parentNode.insertBefore(this.wdg, target);
		target.parentNode.insertBefore(mkgap(), target);
	}

	this.btn = document.createElement('button');
	this.wdg.appendChild(this.btn);
	this.btn.setAttribute('type', 'zone');
	this.btn.onclick = function () { document.zm_menu.open(this); }

	this.details = document.createElement('div');
	this.wdg.appendChild(this.details);

	this.style = null;
	this.group = null;

	this.setup_button();
	this.setup_details();

	this.tbl_tr = mktable(this.wdg);

	this.positions = new Array();
}

Zone.prototype.get_by_name = function (name)
{
	if (name == '')
		name = null;

	for (var i = 0, len = this.positions.length; i < len; i++)
		if (this.positions[i].name == name)
			return this.positions[i];

	return null;
}

Zone.prototype.setup_button = function ()
{
	// update zone index
	this.btn.setAttribute('zone_idx', this.idx);

	// update caption
	clean(this.btn);
	this.btn.appendChild(document.createTextNode(document.strs.zone + ' ' + (this.idx + 1)));

	var img = document.createElement('img');
	this.btn.appendChild(img);
	img.src = document.res.btn_arrow;
}

Zone.prototype.setup_details = function ()
{
	clean(this.details);

	var st = this.style != null ? (document.strs.style + ':' + (style_exists(this.style) ? this.style : '<span class="error">' + this.style + '</span>')) : '';

	if (this.group != null)
		st += (st == '' ? '' : ' - ') + document.strs.group + ':' + this.group;

	this.details.innerHTML = st;
}

Zone.prototype.increase_idx = function (increment)
{
	this.idx += increment;
	this.setup_button();
}

Zone.prototype.delete_it = function ()
{
	this.wdg.parentNode.removeChild(this.wdg.nextSibling);
	this.wdg.parentNode.removeChild(this.wdg);
}

Zone.prototype.add_main_body = function ()
{
	var pos = new ZonePosition(this);
	this.positions.push(pos);
	pos.init_main_body();
	this.update_widths();

	// add gap if necessary
	if (this.tbl_tr.firstChild)
		this.tbl_tr.appendChild(mktdgap());

	this.tbl_tr.appendChild(pos.td);

	return pos;
}

Zone.prototype.add_position_start = function (name)
{
	if (!this.tbl_tr.firstChild) {
		this.add_position_end(name);
		return;
	}

	var pos = new ZonePosition(this);
	this.positions.push(pos);
	pos.init_zone_position(name);
	this.tbl_tr.insertBefore(pos.td, this.tbl_tr.firstChild);

	// add gap
	this.tbl_tr.insertBefore(mktdgap(), pos.td.nextSibling);

	this.update_widths();
	return pos;
}

Zone.prototype.add_position_end = function (name)
{
	var pos = new ZonePosition(this);
	this.positions.push(pos);
	pos.init_zone_position(name);

	// add gap if necessary
	if (this.tbl_tr.firstChild)
		this.tbl_tr.appendChild(mktdgap());

	this.tbl_tr.appendChild(pos.td);

	this.update_widths();
	return pos;
}

Zone.prototype.add_position_lr = function (direction, name, offset_pos_name)
{
	if (offset_pos_name == '')
		offset_pos_name = null;

	var pos = new ZonePosition(this);
	pos.init_zone_position(name);

	for (var i = 0, len = this.positions.length; i < len; i++)
		if (this.positions[i].name == offset_pos_name) {
			if (direction == 'left') {
				this.tbl_tr.insertBefore(pos.td, this.positions[i].td);
				this.tbl_tr.insertBefore(mktdgap(), this.positions[i].td);

				this.positions.splice(i, 0, pos);
			}
			else if (direction == 'right') {
				var target = this.positions[i].td.nextSibling;
				if (target != null) {
					target = target.nextSibling;
					this.tbl_tr.insertBefore(pos.td, target);
					this.tbl_tr.insertBefore(mktdgap(), target);
				}
				else {
					this.tbl_tr.appendChild(mktdgap());
					this.tbl_tr.appendChild(pos.td);
				}

				this.positions.splice(i + 1, 0, pos);
			}

			break;
		}

	this.update_widths();
	return pos;
}

Zone.prototype.delete_position = function (pos_name)
{
	for (var i = 0, len = this.positions.length; i < len; i++)
		if (this.positions[i].name == pos_name) {
			// find the previous gap
			var td = this.positions[i].td.previousSibling;

			// if previous gap not found, find the next gap
			if (td == null)
				td = this.positions[i].td.nextSibling;

			// if a gap was found, delete it
			if (td != null)
				td.parentNode.removeChild(td);

			// delete position
			this.positions[i].td.parentNode.removeChild(this.positions[i].td);
			this.positions.splice(i, 1);

			this.update_widths();

			break;
		}
}

Zone.prototype.has_main_body = function ()
{
	for (var i = 0, len = this.positions.length; i < len; i++)
		if (this.positions[i].is_main_body())
			return true;

	return false;
}

Zone.prototype.cut = function (cut_pos_name)
{
	var result = null;

	for (var i = 0, len = this.positions.length; i < len; i++)
		if (this.positions[i].name == cut_pos_name) {
			if (this.positions[i].td.previousSibling)
				this.tbl_tr.removeChild(this.positions[i].td.previousSibling);
			else if (this.positions[i].td.nextSibling)
				this.tbl_tr.removeChild(this.positions[i].td.nextSibling);

			result = this.positions[i];
			this.positions.splice(i, 1);

			break;
		}

	return result;
}

Zone.prototype.paste = function (type, cut_zone, cut_pos_name, target_pos_name)
{
	if (cut_pos_name == '')
		cut_pos_name = null;

	var cut_pos = cut_zone.cut(cut_pos_name);
	if (cut_pos == null) {
		alert(document.strs.cut_pos_missing);
		return;
	}
	cut_zone.update_widths();

	if (type == 'left' || type == 'right') {
		if (target_pos_name == '')
			target_pos_name = null;
	
		var target_pos = null;
		var target_idx = -1;
		for (var i = 0, len = this.positions.length; i < len; i++) {
			if (this.positions[i].name == target_pos_name) {
				target_idx = i;
				target_pos = this.positions[i];
				break;
			}
		}
	}

	if (type == 'right' && target_idx == this.positions.length - 1)
		type = 'end';

	if (type != 'end' && this.positions.length == 0)
		type = 'end';

	switch (type) {
		case 'start':
			this.tbl_tr.insertBefore(mktdgap(), this.tbl_tr.firstChild);
			this.tbl_tr.insertBefore(cut_pos.td, this.tbl_tr.firstChild);
			this.positions.unshift(cut_pos);
			break;

		case 'end':
			if (this.tbl_tr.firstChild)
				this.tbl_tr.appendChild(mktdgap());
			this.tbl_tr.appendChild(cut_pos.td);
			this.positions.push(cut_pos);

			break;

		case 'left':
			this.tbl_tr.insertBefore(cut_pos.td, target_pos.td);
			this.tbl_tr.insertBefore(mktdgap(), target_pos.td);
			this.positions.splice(target_idx, 0, cut_pos);
			break;

		case 'right':
			var target_wdg = target_pos.td.nextSibling.nextSibling;
			this.tbl_tr.insertBefore(cut_pos.td, target_wdg);
			this.tbl_tr.insertBefore(mktdgap(), target_wdg);
			this.positions.splice(target_idx + 1, 0, cut_pos);
			break;
	}

	cut_pos.btn.setAttribute('zone_idx', this.idx);

	this.update_widths();
}

Zone.prototype.update_widths = function ()
{
	var width = Math.floor(100 / this.positions.length);
	for (var i = 0, len = this.positions.length; i < len; i++)
		this.positions[i].td.setAttribute('width', width + '%');
}
